@@ -40,9 +40,6 @@ int HttpLogSource::initAccessLogs(const XmlNode *pRoot,
 {
     int ret = 0;
     ConfigCtx currentCtx("accesslog");
-    //const XmlNode *p0 = pRoot->getChild( "logging", 1 );
-    //const XmlNode *pNode1 = pRoot->getChild("accessLog");
-
     const XmlNodeList *pList = pRoot->getChildren("accessLog");
     if (pList)
     {
@@ -58,15 +55,14 @@ int HttpLogSource::initAccessLogs(const XmlNode *pRoot,
     return ret;
 }
     
-    
+
 int HttpLogSource::initAccessLog(const XmlNode *pNode,
                                  int setDebugLevel, int inList)
 {
     ConfigCtx currentCtx("accesslog");
-    
-    const XmlNode *pNode1 = (pNode ? pNode->getChild("accessLog") : NULL);
-    if (inList)
-        pNode1 = pNode;
+    const XmlNode *pNode1 = pNode;
+    if (!inList)
+        pNode1 = (pNode ? pNode->getChild("accessLog") : NULL);
     
     if (pNode1 == NULL)
     {
