@@ -38,6 +38,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include "mainserverconfig.h"
 
 //#define TEST_OUTPUT_PLAIN_CONF
 
@@ -63,6 +64,7 @@ plainconfKeywords plainconf::sKeywords[] =
     {"allowbrowse",                              NULL},
     {"allowdirectaccess",                        NULL},
     {"allowedhosts",                             NULL},
+    {"allowedrobothits",                         NULL},
     {"allowsetuid",                              NULL},
     {"allowsymbollink",                          NULL},
     {"authname",                                 NULL},
@@ -80,6 +82,7 @@ plainconfKeywords plainconf::sKeywords[] =
     {"banperiod",                                NULL},
     {"base",                                     NULL},
     {"binding",                                  NULL},
+    {"botwhitelist",                             NULL},
     {"brstaticcompresslevel",                    NULL},
     {"byteslog",                                 NULL},
     {"cacertfile",                               NULL},
@@ -90,6 +93,7 @@ plainconfKeywords plainconf::sKeywords[] =
     {"certfile3",                                NULL},
     {"cgidsock",                                 NULL},
     {"cgirlimit",                                NULL},
+    {"cgroups",                                  NULL},
     {"checksymbollink",                          NULL},
     {"chrootmode",                               NULL},
     {"chrootpath",                               NULL},
@@ -205,6 +209,7 @@ plainconfKeywords plainconf::sKeywords[] =
     {"logreferer",                               NULL},
     {"loguseragent",                             NULL},
     {"ls_enabled",                               NULL},
+    {"lsrecaptcha",                              NULL},
     {"map",                                      NULL},
     {"maxcachedfilesize",                        NULL},
     {"maxcachesize",                             NULL},
@@ -221,6 +226,7 @@ plainconfKeywords plainconf::sKeywords[] =
     {"maxreqheadersize",                         NULL},
     {"maxrequrllen",                             NULL},
     {"maxsslconnections",                        NULL},
+    {"maxtries",                                 NULL},
     {"member",                                   NULL},
     {"memhardlimit",                             NULL},
     {"memsoftlimit",                             NULL},
@@ -253,6 +259,7 @@ plainconfKeywords plainconf::sKeywords[] =
     {"realm",                                    NULL},
     {"realmlist",                                NULL},//!!
     {"redirecturi",                              NULL},
+    {"regconnlimit",                             NULL},
     {"required",                                 NULL},
     {"requiredpermissionmask",                   NULL},
     {"respbuffer",                               NULL},
@@ -265,10 +272,13 @@ plainconfKeywords plainconf::sKeywords[] =
     {"runonstartup",                             NULL},
     {"scripthandler",                            NULL},
     {"scripthandlerlist",                        NULL},
+    {"secretkey",                                NULL},
     {"secure",                                   NULL},
     {"securedconn",                              NULL},
     {"security",                                 NULL},
     {"servername",                               NULL},
+    {"sitekey",                                  NULL},
+    {"sslconnlimit",                             NULL},
     {"ssldefaultcafile",                         NULL},
     {"ssldefaultcapath",                         NULL},
     {"sslenablemulticerts",                      NULL},
@@ -406,7 +416,9 @@ plainconfKeywords plainconf::sKeywords[] =
     {"binpath", NULL},
     {"apptype", NULL},
     {"startupfile", NULL},
-    
+    {"appserverenv", NULL},
+    {"enablelve",  NULL},
+    {"cpuaffinity", NULL},
     
 };
 
@@ -1127,7 +1139,11 @@ void plainconf::getIncludeFile(const char *curDir, const char *orgFile,
 //Example: co -u1.1 httpd_config.conf
 void plainconf::checkInFile(const char *path)
 {
-    if (access(path, 0) == -1)
+    if (MainServerConfig::getInstance().getDisableWebAdmin())
+        return ;
+    
+    struct stat sb0;
+    if (stat(path, &sb0) == -1)
         return ;
 
     //Backup file abd checkin and out
@@ -1135,36 +1151,40 @@ void plainconf::checkInFile(const char *path)
     strcpy(new_path, path);
     strcat(new_path, "0");
 
+    int ret;
+    struct stat sb;
     AutoStr2 buf;
-    buf.setStr("cp \"");
-    buf.append(path, strlen(path));
-    buf.append("\" \"", 3);
-    buf.append(new_path, strlen(new_path));
-    buf.append("\"", 1);
-    int ret = system(buf.c_str());
-
-    if (ret != 0)
+    if (stat(new_path, &sb) == -1 ||
+        (sb.st_mtime < sb0.st_mtime && sb0.st_size != sb.st_size))
     {
-        logToMem(LOG_LEVEL_INFO, "Failed to backup the conf file %s, ret %d.",
-                 path, ret);
-        return ;
+        buf.setStr("cp -f \"");
+        buf.append(path, strlen(path));
+        buf.append("\" \"", 3);
+        buf.append(new_path, strlen(new_path));
+        buf.append("\"", 1);
+        ret = system(buf.c_str());
+        if (ret != 0)
+        {
+            logToMem(LOG_LEVEL_INFO, "Failed to backup the conf file %s, ret %d.",
+                    path, ret);
+            return ;
+        }
+        
+        buf.setStr("ci -l -q -t-\"");
+        buf.append(new_path, strlen(new_path));
+        buf.append("\" -mUpdate \"", 12);
+        buf.append(new_path, strlen(new_path));
+        buf.append("\" >/dev/null 2>&1", 17);
+        ret = system(buf.c_str());
+
+        if (ret == 0)
+            logToMem(LOG_LEVEL_INFO, "RCS checkin config file %s OK.", new_path);
+        else
+            logToMem(LOG_LEVEL_INFO,
+                    "Failed to RCS checkin conf file %s, ret %d, error(%s). "
+                    "Org command is %s.", new_path, ret, strerror(errno), buf.c_str());
     }
-
-    buf.setStr("ci -l -q -t-\"");
-    buf.append(new_path, strlen(new_path));
-    buf.append("\" -mUpdate \"", 12);
-    buf.append(new_path, strlen(new_path));
-    buf.append("\" >/dev/null 2>&1", 17);
-    ret = system(buf.c_str());
-
-    if (ret == 0)
-        logToMem(LOG_LEVEL_INFO, "RCS checkin config file %s OK.", new_path);
-    else
-        logToMem(LOG_LEVEL_INFO,
-                 "Failed to RCS checkin conf file %s, ret %d, error(%s). "
-                 "Org command is %s.", new_path, ret, strerror(errno), buf.c_str());
-
-    unlink(new_path);
+    
 }
 
 
@@ -1250,10 +1270,8 @@ void plainconf::loadConfFile(const char *path)
         char sMultiLineModeSign[MAX_MULLINE_SIGN_LENGTH] = {0};
         size_t  nMultiLineModeSignLen = 0;  //>0 is mulline mode
 
-        while (!feof(fp))
+        while (fgets(sLine, MAX_LINE_LENGTH, fp))
         {
-            sLine[0] = 0x00;
-            fgets(sLine, MAX_LINE_LENGTH, fp);
             ++lineNumber;
             p = sLine;
 
@@ -1289,7 +1307,6 @@ void plainconf::loadConfFile(const char *path)
                 continue;
 
             AutoStr2 pathInclude;
-
             if (isInclude(p, pathInclude))
             {
                 char achBuf[512] = {0};
@@ -1354,13 +1371,6 @@ void plainconf::loadConfFile(const char *path)
 XmlNode *plainconf::parseFile(const char *configFilePath,
                               const char *rootTag)
 {
-#ifdef TEST_OUTPUT_PLAIN_CONF
-    char *tmp = (char *)new char[5 * 1024 * 1024];
-
-    if (tmp)
-        delete []tmp;
-
-#endif
     XmlNode *rootNode = new XmlNode;
     const char *attr = NULL;
     rootNode->init(rootTag, &attr);
